@@ -4,21 +4,21 @@
 This module provides low-level API for Tarantool
 '''
 
-import six
+import os
 import time
 import errno
+import socket
+
 import ctypes
 import ctypes.util
-import socket
-import msgpack
-import os
-
 try:
     from ctypes import c_ssize_t
 except ImportError:
     from ctypes import c_longlong as c_ssize_t
 
-import tarantool
+import msgpack
+
+import tarantool.error
 from tarantool.response import Response
 from tarantool.request import (
     Request,
@@ -44,19 +44,26 @@ from tarantool.const import (
     REQUEST_TYPE_OK,
     REQUEST_TYPE_ERROR,
     IPROTO_GREETING_SIZE,
-    ENCODING_DEFAULT,
     ITERATOR_EQ,
     ITERATOR_ALL
 )
 from tarantool.error import (
     NetworkError,
-    # DatabaseError,
+    DatabaseError,
+    InterfaceError,
+    SchemaError,
     NetworkWarning,
     SchemaReloadException,
     warn
 )
 from tarantool.schema import Schema
-from tarantool.utils import check_key, greeting_decode, version_id
+from tarantool.utils import (
+    check_key,
+    greeting_decode,
+    version_id,
+    string_types,
+    ENCODING_DEFAULT,
+)
 
 
 class Connection(object):
@@ -69,10 +76,10 @@ class Connection(object):
     (insert/delete/update/select).
     '''
     Error = tarantool.error
-    DatabaseError = tarantool.error.DatabaseError
-    InterfaceError = tarantool.error.InterfaceError
-    SchemaError = tarantool.error.SchemaError
-    NetworkError = tarantool.error.NetworkError
+    DatabaseError = DatabaseError
+    InterfaceError = InterfaceError
+    SchemaError = SchemaError
+    NetworkError = NetworkError
 
     def __init__(self, host, port,
                  user=None,
@@ -81,7 +88,8 @@ class Connection(object):
                  reconnect_max_attempts=RECONNECT_MAX_ATTEMPTS,
                  reconnect_delay=RECONNECT_DELAY,
                  connect_now=True,
-                 encoding=ENCODING_DEFAULT):
+                 encoding=ENCODING_DEFAULT,
+                 call_16=False):
         '''
         Initialize a connection to the server.
 
@@ -114,6 +122,7 @@ class Connection(object):
         self.connected = False
         self.error = True
         self.encoding = encoding
+        self.call_16 = call_16
         if connect_now:
             self.connect()
 
@@ -125,6 +134,12 @@ class Connection(object):
         self._socket = None
 
     def connect_basic(self):
+        if self.host == None:
+            self.connect_unix()
+        else:
+            self.connect_tcp()
+
+    def connect_tcp(self):
         '''
         Create connection to the host and port specified in __init__().
         :raise: `NetworkError`
@@ -141,6 +156,23 @@ class Connection(object):
             self.connected = False
             raise NetworkError(e)
 
+    def connect_unix(self):
+        '''
+        Create connection to the host and port specified in __init__().
+        :raise: `NetworkError`
+        '''
+
+        try:
+            # If old socket already exists - close it and re-create
+            self.connected = True
+            if self._socket:
+                self._socket.close()
+            self._socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+            self._socket.connect(self.port)
+        except socket.error as e:
+            self.connected = False
+            raise NetworkError(e)
+
     def handshake(self):
         greeting_buf = self._recv(IPROTO_GREETING_SIZE)
         greeting = greeting_decode(greeting_buf)
@@ -337,7 +369,7 @@ class Connection(object):
         if len(args) == 1 and isinstance(args[0], (list, tuple)):
             args = args[0]
 
-        request = RequestCall(self, func_name, args)
+        request = RequestCall(self, func_name, args, self.call_16)
         response = self._send_request(request)
         return response
 
@@ -375,7 +407,7 @@ class Connection(object):
 
         :rtype: `Response` instance
         '''
-        if isinstance(space_name, six.string_types):
+        if isinstance(space_name, string_types):
             space_name = self.schema.get_space(space_name).sid
         request = RequestReplace(self, space_name, values)
         return self._send_request(request)
@@ -429,6 +461,15 @@ class Connection(object):
                 if state == JoinState.Done:
                     return
 
+    def _ops_process(self, space, update_ops):
+        new_ops = []
+        for op in update_ops:
+            if isinstance(op[1], string_types):
+                op = list(op)
+                op[1] = self.schema.get_field(space, op[1])['id']
+            new_ops.append(op)
+        return new_ops
+
     def join(self, server_uuid):
         self._opt_reconnect()
         if self.version_id < version_id(1, 7, 0):
@@ -459,7 +500,7 @@ class Connection(object):
 
         :rtype: `Response` instance
         '''
-        if isinstance(space_name, six.string_types):
+        if isinstance(space_name, string_types):
             space_name = self.schema.get_space(space_name).sid
         request = RequestInsert(self, space_name, values)
         return self._send_request(request)
@@ -480,9 +521,9 @@ class Connection(object):
         index_name = kwargs.get("index", 0)
 
         key = check_key(key)
-        if isinstance(space_name, six.string_types):
+        if isinstance(space_name, string_types):
             space_name = self.schema.get_space(space_name).sid
-        if isinstance(index_name, six.string_types):
+        if isinstance(index_name, string_types):
             index_name = self.schema.get_index(space_name, index_name).iid
         request = RequestDelete(self, space_name, index_name, key)
         return self._send_request(request)
@@ -551,10 +592,11 @@ class Connection(object):
         '''
         index_name = kwargs.get("index", 0)
 
-        if isinstance(space_name, six.string_types):
+        if isinstance(space_name, string_types):
             space_name = self.schema.get_space(space_name).sid
-        if isinstance(index_name, six.string_types):
+        if isinstance(index_name, string_types):
             index_name = self.schema.get_index(space_name, index_name).iid
+        op_list = self._ops_process(space_name, op_list)
         request = RequestUpsert(self, space_name, index_name, tuple_value,
                                 op_list)
         return self._send_request(request)
@@ -625,10 +667,11 @@ class Connection(object):
         index_name = kwargs.get("index", 0)
 
         key = check_key(key)
-        if isinstance(space_name, six.string_types):
+        if isinstance(space_name, string_types):
             space_name = self.schema.get_space(space_name).sid
-        if isinstance(index_name, six.string_types):
+        if isinstance(index_name, string_types):
             index_name = self.schema.get_index(space_name, index_name).iid
+        op_list = self._ops_process(space_name, op_list)
         request = RequestUpdate(self, space_name, index_name, key, op_list)
         return self._send_request(request)
 
@@ -707,9 +750,9 @@ class Connection(object):
         # tuples)
         key = check_key(key, select=True)
 
-        if isinstance(space_name, six.string_types):
+        if isinstance(space_name, string_types):
             space_name = self.schema.get_space(space_name).sid
-        if isinstance(index_name, six.string_types):
+        if isinstance(index_name, string_types):
             index_name = self.schema.get_index(space_name, index_name).iid
         request = RequestSelect(self, space_name, index_name, key, offset,
                                 limit, iterator_type)
